@@ -1,6 +1,6 @@
 package ShardedKV::Storage::MySQL;
 {
-  $ShardedKV::Storage::MySQL::VERSION = '0.11';
+  $ShardedKV::Storage::MySQL::VERSION = '0.12';
 }
 use Moose;
 # ABSTRACT: MySQL storage backend for ShardedKV
@@ -33,12 +33,18 @@ has '_mysql_connection' => (
 
 sub _make_master_conn {
   my $self = shift;
+  my $logger = $self->logger;
+  $logger->debug("Getting master connection") if $logger;
   my $dbh = $self->mysql_master_connector->();
   if ($dbh) {
+    $logger->debug("Get master connection") if $logger;
     $dbh->{RaiseError} = 1;
     $dbh->{PrintError} = 0;
     #$dbh->{AutoCommit} = 1;
   }
+  else {
+    $logger->warn("Failed to get master connection") if $logger;
+  }
   return $dbh;
 }
 
@@ -58,7 +64,19 @@ has 'key_col_name' => (
 
 has 'key_col_type' => (
   is => 'ro',
-  default => "CHAR(16) NOT NULL",
+  default => "VARBINARY(16) NOT NULL",
+);
+
+
+has 'auto_increment_col_name' => (
+  is => 'ro',
+  default => 'id',
+);
+
+
+has 'auto_increment_col_type' => (
+  is => 'ro',
+  default => "BIGINT UNSIGNED NOT NULL AUTO_INCREMENT",
 );
 
 
@@ -108,12 +126,14 @@ has '_get_query' => (
   lazy => 1,
   builder => '_make_get_query',
 );
+
 has '_set_query' => (
   is => 'ro',
   isa => 'Str',
   lazy => 1,
   builder => '_make_set_query',
 );
+
 has '_delete_query' => (
   is => 'ro',
   isa => 'Str',
@@ -144,7 +164,12 @@ sub _make_get_query {
   my $tbl = $self->table_name;
   my ($key_col, $v_cols) = map $self->$_, qw(key_col_name value_col_names);
   my $v_col_str = join ',', @$v_cols;
-  return qq{SELECT $v_col_str FROM $tbl WHERE $key_col = ? LIMIT 1};
+  my $q = qq{SELECT $v_col_str FROM $tbl WHERE $key_col = ? LIMIT 1};
+
+  my $logger = $self->{logger};
+  $logger->debug("Generated the following get-query:\n$q") if $logger;
+
+  return $q;
 }
 
 sub _make_set_query {
@@ -161,6 +186,10 @@ sub _make_set_query {
     ON DUPLICATE KEY UPDATE
     $vcol_assign_str
   };
+
+  my $logger = $self->{logger};
+  $logger->debug("Generated the following set-query:\n$q") if $logger;
+
   return $q;
 }
 
@@ -169,15 +198,23 @@ sub _make_delete_query {
   $self->_number_of_params; # prepopulate
   my $tbl = $self->table_name;
   my $key_col = $self->key_col_name;
-  return qq{DELETE FROM $tbl WHERE $key_col = ? LIMIT 1};
+  my $q = qq{DELETE FROM $tbl WHERE $key_col = ? LIMIT 1};
+
+  my $logger = $self->{logger};
+  $logger->debug("Generated the following delete-query:\n$q") if $logger;
+
+  return $q;
 }
 
 sub prepare_table {
   my $self = shift;
   $self->_number_of_params; # prepopulate
   my $tbl = $self->table_name;
-  my ($key_col, $key_type, $v_cols, $v_types)
-    = map $self->$_, qw(key_col_name key_col_type value_col_names value_col_types);
+  my ($key_col, $key_type, $ainc_col, $ainc_type, $v_cols, $v_types)
+    = map $self->$_,
+      qw(key_col_name key_col_type
+         auto_increment_col_name auto_increment_col_type
+         value_col_names value_col_types);
   my @vcoldefs = map "$v_cols->[$_] $v_types->[$_]", 0..$#$v_cols;
   my $vcol_str = join ",\n", @vcoldefs;
   my $extra_indexes = $self->extra_indexes;
@@ -187,14 +224,29 @@ sub prepare_table {
   else {
     $extra_indexes = ",\n$extra_indexes";
   }
+  my $pk;
+  my $ainc_col_spec = '';
+  if (defined $ainc_col) {
+    $pk = "PRIMARY KEY($ainc_col),\n"
+          . "UNIQUE KEY ($key_col)";
+    $ainc_col_spec = "$ainc_col $ainc_type,";
+  }
+  else {
+    $pk = "PRIMARY KEY($key_col)";
+  }
   my $q = qq{
       CREATE TABLE IF NOT EXISTS $tbl (
+        $ainc_col_spec
         $key_col $key_type,
         $vcol_str,
-        PRIMARY KEY($key_col)
+        $pk
         $extra_indexes
       ) ENGINE=InnoDb
   };
+
+  my $logger = $self->logger;
+  $logger->info("Creating shard storage table:\n$q") if $logger;
+
   $self->get_master_dbh->do($q);
 }
 
@@ -202,6 +254,10 @@ sub prepare_table {
 # a cached connection.
 sub refresh_connection {
   my $self = shift;
+
+  my $logger = $self->{logger};
+  $logger->info("Refreshing mysql connection") if $logger;
+
   delete $self->{_mysql_connection};
   return $self->_mysql_connection;
 }
@@ -216,6 +272,7 @@ sub get_master_dbh {
   if (not defined $master_dbh) {
     die "Failed to get connection to mysql!";
   }
+
   return $master_dbh;
 }
 
@@ -257,7 +314,7 @@ sub set {
   my ($self, $key, $value_ref) = @_;
 
   Carp::croak("Need exactly " . ($self->{_number_of_params}-1) . " values, got " . scalar(@$value_ref))
-    if not scalar(@$value_ref) == $self->{_number_of_params}-1;
+    if not scalar(@$value_ref) == $self->_number_of_params-1;
 
   my $rv = $self->_run_sql('do', $self->_set_query, undef, $key, @$value_ref);
   return $rv ? 1 : 0;
@@ -282,7 +339,7 @@ ShardedKV::Storage::MySQL - MySQL storage backend for ShardedKV
 
 =head1 VERSION
 
-version 0.11
+version 0.12
 
 =head1 SYNOPSIS
 
@@ -332,7 +389,8 @@ Must be supplied at object creation.
 
 The name of the column to be used for the key.
 If C<ShardedKV::Storage::MySQL> creates the shard table for you, then
-this column is also used as the primary key.
+this column is also used as the primary key unless
+C<auto_increment_col_name> is set (see below).
 
 There can only be one key column.
 
@@ -342,7 +400,29 @@ Defaults to 'keystr'.
 
 The MySQL type of the key column.
 
-Defaults to 'CHAR(16) NOT NULL'.
+Defaults to 'VARBINARY(16) NOT NULL'.
+
+=head2 auto_increment_col_name
+
+The name of the column to be used for the auto-increment pimrary key.
+This is a virtually unused (by ShardedKV) column that, IF DEFINED, will
+be used as an auto-increment primary key. It is not the column used to
+fetch rows by, but rather facilitates faster insertion of new records
+by allowing append instead of insertion at random order within the PK
+tree.
+
+If C<ShardedKV::Storage::MySQL> creates the shard table for you, then
+this column is also used as the primary key.
+
+There can only be one auto-increment key column.
+
+Defaults to 'id'.
+
+=head2 auto_increment_col_type
+
+The MySQL type of the auto increment column.
+
+Defaults to 'BIGINT UNSIGNED NOT NULL AUTO_INCREMENT'.
 
 =head2 value_col_names
 
